/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CreateTextualVersionCommand.as 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.editor.command
{
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.utils.getQualifiedClassName;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.event.*;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.textual.ITextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.IVisualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.editor.business.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.event.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.RuleEditorModelLocator;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.domain.RuleEditorModel;

/**
* Create a textual version of a rule package and edit it
* 
* @author Julien Nicoulaud
*/
public class CreateTextualVersionCommand implements ICommand
{

    // =========================================================================
    // Execution
    // =========================================================================
    
    /**
    * Execute the command
    */
    public function execute(event:CairngormEvent):void
    {
        // Retrieve the component model
        var model:RuleEditorModel = 
            RuleEditorModelLocator.getInstance().ruleEditorModel;
        
        // Get the target rule package
        var synchronizedRulePackage : SynchronizedRulePackage =
            (event as SynchronizedRulePackageEvent).rulePackage;
                            
        // Dispatch a notification to Kerneos
        CairngormEventDispatcher.getInstance().dispatchEvent(
            new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION + model.componentID,
                                         (getQualifiedClassName(this).split("::"))[1] + ": Creating a textual version of \"" +
                                         synchronizedRulePackage.rulePackage.systemName + "\"",
                                         KerneosNotificationEvent.DEBUG)
        );
        
        // Make sure it is a visual rule package
        if (!synchronizedRulePackage.rulePackage is IVisualRulePackageVO) {
            throw new Error('Can only create textual versions of visual rule packages');
        } else {

        	// Create the new rule package
        	var oldRulePackage : IVisualRulePackageVO = synchronizedRulePackage.rulePackage as IVisualRulePackageVO;
        	var newRulePackage : ITextualRulePackageVO = oldRulePackage.getTextualPackage();
        	newRulePackage.displayName = oldRulePackage.displayName + ' (Textual Version)';
            
            // Add it to the edited objects
            model.editedRulePackages.addItem(new SynchronizedRulePackage(newRulePackage,
                                                                         SynchronizedRulePackageState.MODIFIED));
            
        }
    }

}
}
