/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleEditorModelLocator.as 4905 2009-08-13 13:35:13Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.editor.model
{

import com.adobe.cairngorm.model.ModelLocator;

import org.ow2.jasmine.monitoring.eos.rules.editor.business.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.domain.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.presentation.*;
	
/**
* Cairngorm Model Locator for the rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class RuleEditorModelLocator implements ModelLocator 
{
    // =========================================================================
    // Variables
    // =========================================================================
            
    /**
    * Singleton instance of this locator
    */
	private static var instance:RuleEditorModelLocator;
	
	
    // Models
    
    /**
    * Rule engine dashboard domain model
    */
    public var ruleEditorModel:RuleEditorModel;
    
    /**
    * Rule engine dashboard presentation model
    */
    public var ruleEditorPresentationModel:RuleEditorMainViewPM;
    

    // Command delegates unique instances
    
    /**
    * SaveRulePackage delegate unique instance
    */
    private var saveRulePackageDelegate:ISaveRulePackageDelegate = null;

    /**
    * GetRulePackage delegate unique instance
    */
    private var getRulePackageDelegate:IGetRulePackageDelegate = null;

    /**
    * CompileRulePackage delegate unique instance
    */
    private var compileRulePackageDelegate:ICompileRulePackageDelegate = null;

    /**
    * CheckSystemNameAvailability delegate unique instance
    */
    private var checkSystemNameAvailabilityDelegate:ICheckSystemNameAvailabilityDelegate = null;

    /**
    * ExportRulePackagesAsXML delegate unique instance
    */
    private var exportRulePackagesAsXMLDelegate:IExportRulePackagesAsXMLDelegate = null;

    /**
    * ImportRulePackage delegate unique instance
    */
    private var importRulePackageDelegate:IImportRulePackageDelegate = null;
    
    
    // =========================================================================
    // Construction and singleton instance access
    // =========================================================================
    
    /**
    * Constructor
    */
	public function RuleEditorModelLocator()
	{
		// Instanciate the singleton
		if(instance!=null) {
			throw new Error("Error: Singletons can only be" + 
							"instantiated via getInstance() method!");
		}
		RuleEditorModelLocator.instance = this;
	}
	
	/**
	* EngineDashboardModelLocator singleton instance getter
	*/
	public static function getInstance():RuleEditorModelLocator
	{
		if(instance==null){
            
            // Instanciate the ModelLocator
            instance = new RuleEditorModelLocator();
            
            // Instanciate the models
            instance.ruleEditorModel = new RuleEditorModel();
            instance.ruleEditorPresentationModel = new RuleEditorMainViewPM(instance.ruleEditorModel);
            
        }
		return instance;
	}
	
	
    // =========================================================================
    // Delegates getters
    // =========================================================================
    
    /**
    * "Save rule package" delegate getter
    */
    public function getSaveRulePackageDelegate():ISaveRulePackageDelegate {
        if (this.saveRulePackageDelegate == null) {
            this.saveRulePackageDelegate = new SaveRulePackageDelegate();
        }
        return this.saveRulePackageDelegate;
    }

    /**
    * "Get rule package" delegate getter
    */
    public function getGetRulePackageDelegate():IGetRulePackageDelegate {
        if (this.getRulePackageDelegate == null) {
            this.getRulePackageDelegate = new GetRulePackageDelegate();
        }
        return this.getRulePackageDelegate;
    }

    /**
    * "Compile rule package" delegate getter
    */
    public function getCompileRulePackageDelegate():ICompileRulePackageDelegate {
        if (this.compileRulePackageDelegate == null) {
            this.compileRulePackageDelegate = new CompileRulePackageDelegate();
        }
        return this.compileRulePackageDelegate;
    }

    /**
    * "Check name availability" delegate getter
    */
    public function getCheckSystemNameAvailabilityDelegate():ICheckSystemNameAvailabilityDelegate {
        if (this.checkSystemNameAvailabilityDelegate == null) {
            this.checkSystemNameAvailabilityDelegate = new CheckSystemNameAvailabilityDelegate();
        }
        return this.checkSystemNameAvailabilityDelegate;
    }

    /**
    * "Export rule packages as XML" delegate getter
    */
    public function getExportRulePackagesAsXMLDelegate():IExportRulePackagesAsXMLDelegate {
        if (this.exportRulePackagesAsXMLDelegate == null) {
            this.exportRulePackagesAsXMLDelegate = new ExportRulePackagesAsXMLDelegate();
        }
        return this.exportRulePackagesAsXMLDelegate;
    }

    /**
    * "Import rule package" delegate getter
    */
    public function getImportRulePackageDelegate():IImportRulePackageDelegate {
        if (this.importRulePackageDelegate == null) {
            this.importRulePackageDelegate = new ImportRulePackageDelegate();
        }
        return this.importRulePackageDelegate;
    }
}
}
