/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RuleEditorModel.as 5621 2009-11-08 17:42:06Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.editor.model.domain
{

import mx.collections.ArrayCollection;
import mx.utils.UIDUtil;

import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.AbstractRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.IRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.textual.TextualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.VisualRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.editor.vo.*;

/**
* Cairngorm Model for the rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class RuleEditorModel
{
	
    // =========================================================================
    // Variables
    // =========================================================================

    /**
    * The unique ID of this component
    * 
    * @internal
    *   Used to prevent a Cairngorm issue: when a command event is dispatched,
    * every controller that registered this event type receives it, even if
    * located in another module. To prevent this from happening and triggering
    * multiple severe unexpected concurrence bugs, each event dispatched is
    * postfixed with this unique ID.
    */
    public var componentID:String = UIDUtil.createUID();
        
    /**
    * The name of the service used by the component
    */
    public var serviceName:String;

    /**
    * The managed rule engine instance name
    */
    public var ruleEngineName:String;

    /**
    * The visual edition mode state.
    */
    public var allowVisualEdition:Boolean = true;
	
    /**
    * The edited synchronized rule packages
    */
    [ArrayElementType("org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage")] 
    public var editedRulePackages : ArrayCollection = new ArrayCollection;
    
    /**
    * The package containing the visual element types
    */
    public static var visualElementTypesPackage : String
        = 'org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.';
    
    /**
    * The allowed element types in visual edition mode
    */
    public var allowedVisualElementTypes : ArrayCollection
        = new ArrayCollection(['RHSCodeSnippetVO',
                               'LHSCodeSnippetVO',
                               'LogicalExpressionVO']);
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Create a "ready to be edited" textual rule package object and add it
    * to the edited objects
    */
    public function editNewTextualRulePackage():void
    {
        var newRulePackage : IRulePackageVO = createNewTextualRulePackage();
        editedRulePackages.addItem(
           new SynchronizedRulePackage(newRulePackage,
                                       SynchronizedRulePackageState.SYNCHRONIZED)
        );
    }
    
    /**
    * Create a "ready to be edited" visual rule package object and add it
    * to the edited objects
    */
    public function editNewVisualRulePackage():void
    {
        var newRulePackage : IRulePackageVO = createNewVisualRulePackage();
        editedRulePackages.addItem(
           new SynchronizedRulePackage(newRulePackage,
                                       SynchronizedRulePackageState.SYNCHRONIZED)
        );
    }
    
    /**
    * Create a new textual rule package
    */
    public function createNewTextualRulePackage():TextualRulePackageVO
    {
        var date:Date = new Date();
        var newRulePackage : TextualRulePackageVO = new TextualRulePackageVO();
        newRulePackage.displayName = "New rule package (" + date.getDate()
                                                        + "/" + (date.getMonth()+1)
                                                        + "/" + date.getFullYear()
                                                        + " - " + date.getHours()
                                                        + ":" + date.getMinutes()
                                                        + ":" + date.getSeconds() + ")";
        newRulePackage.ruleEngineInstanceName = ruleEngineName;
        newRulePackage.state = AbstractRulePackageVO.DEACTIVATED;
        newRulePackage.description =
           "Here is a rule package to start." +
           "\nEnjoy !";
        newRulePackage.content = 
            "package org.ow2.jasmine.rules.packages.NewEmptyRulePackage" +
            "\n" +
            "\nrule \"Empty rule\"" +
            "\n    no-loop false" +
            "\n    when" +
            "\n        // Let's check some facts !" +
            "\n    then" +
            "\n        // Let's act !" +
            "\nend\n";
        return newRulePackage;
    }
        
    /**
    * Create a new visual rule package
    */
    public function createNewVisualRulePackage():VisualRulePackageVO
    {
        var date:Date = new Date();
        var newRulePackage : VisualRulePackageVO = new VisualRulePackageVO();
        newRulePackage.displayName = "New rule package (" + date.getDate()
                                                        + "/" + (date.getMonth()+1)
                                                        + "/" + date.getFullYear()
                                                        + " - " + date.getHours()
                                                        + ":" + date.getMinutes()
                                                        + ":" + date.getSeconds() + ")";
        newRulePackage.ruleEngineInstanceName = ruleEngineName;
        newRulePackage.state = AbstractRulePackageVO.DEACTIVATED;
        newRulePackage.description =
           "Enter your description here !";
        //var newRule : IRuleVO = new RuleVO();
        //newRule.imports = newRulePackage.imports;
        //newRule.name = "My first rule";
        /*var newLHSElement : LogicalExpressionVO = new LogicalExpressionVO();
        newRule.lhs.addItem(newLHSElement);*/
        //var newLHSElement : LHSCodeSnippetVO = new LHSCodeSnippetVO();
        //newLHSElement.content = "// Let's check some facts !";
        //newLHSElement.imports = newRulePackage.imports;
        //newRule.lhs.addItem(newLHSElement);
        //var newRHSElement : RHSCodeSnippetVO = new RHSCodeSnippetVO();
        //newRHSElement.content = "// Let's act !";
        //newRHSElement.imports = newRulePackage.imports;
        //newRule.rhs.addItem(newRHSElement);
        //newRulePackage.rules.addItem(newRule);
        return newRulePackage;
    }
    
    /**
    * Test if a given rule package is already edited
    */
    public function getEditedRulePackage(rulePackageSystemName:String):SynchronizedRulePackage
    {
        // Loop on the edited objects searching for this one
        for each (var editedRulePackage : SynchronizedRulePackage in editedRulePackages) {
            if(editedRulePackage.rulePackage.systemName == rulePackageSystemName) {
                return editedRulePackage;
            }
        }
        return null;
    }
    
    /**
    * Remove a rule package from the edited objects given its system name
    */
    public function removeRulePackageByName(rulePackageSystemName:String):Boolean
    {
        // Loop on the edited objects searching for this one
        for each (var editedRulePackage : SynchronizedRulePackage in editedRulePackages) {
            if(editedRulePackage.rulePackage.systemName == rulePackageSystemName) {
                return removeRulePackage(editedRulePackage);
            }
        }
        return false;
    }
    
    /**
    * Remove a rule package from the edited objects
    */
    public function removeRulePackage(rulePackage:SynchronizedRulePackage):Boolean
    {
        return (editedRulePackages.removeItemAt(
                    editedRulePackages.getItemIndex(rulePackage)
                ) != null);
    }
}
}
